function bf=selectFeatures(X,cLabs,D,doPlot)
% function bestFeaturesIdx=selectFeatures(X,cLabs,D,doPlot)
% 
% Select best features using point-biserial correlation criterion and
% univariate ranking search method.
%
% This is not necessarily a good idea in practice, but serves as an
% illustration.
%
% IN
%   X:      nFeats x nPatterns matrix of column feature vectors
%   cLabs:  nPatterns x 1 vector of class labels
%   D:      scalar indicating how many features to select
%   doPlot: boolean, whether to plot sorted features graph or not
%
% OUT
%   bf:     D x 1 vector of feature indices indicating the best features
%
% v1.0 May 2010 Jonas Richiardi for Advanced fMRI course

% sanity check
[nFeats nPats]=size(X);
if length(cLabs)~=nPats
    error('cLabs length inconsistent with number of patterns');
end
if D>nFeats
    error('D must be smaller or equal to number of features in X');
end

% compute per-feature point-biserial correlation (which is a special
% case of pearson product-moment correlation, so no need to implement
% your own code except for maybe speed gains)
Rpb=zeros(1,nFeats);
for f=1:nFeats
    rpb=corrcoef(X(f,:),cLabs);
    Rpb(f)=abs(rpb(1,2)); % extract correlation coeff from matrix
end

% get ranking
[Rpb_srt,Rpb_srt_idx]=sort(Rpb,'descend');

% extract top D feature indices
bf=Rpb_srt_idx(1:D);

if doPlot==true
    figure; plot(Rpb_srt,'LineWidth',2.5);
    %set(gca,'XTick',1:4:nFeats,'XTickLabel',...
    %    num2strcell(Rpb_srt_idx(1:2:nFeats)),'FontSize',13);
    xlabel('Feature number');
    ylabel('|r_{pb}|'); ylim([0 1]);
    grid on; hold on;
    % D-line
    line([D D],[0 1],'LineStyle','--','LineWidth',2,'Color','r');
end